% M-file 9.6
% CVCrossSynthesis.m
%
% cross-synthesis with channel vocoder

%----- USER DATA -----
[y1,FS] = wavread('moore_guitar'); % reading the two sound files
y2      = wavread('toms_diner');
ly      = min(length(y1), length(y2));% length of the signals
res     = zeros(ly,1); % result signal
r       = 0.99;
lp      = [1, -2*r, +r*r]; % filter used
epsi    = 0.00001;
%----- performing the vocoding or cross synthesis effect -----

% bandpass frequencies:
f0 = 10; % start freq in Hz
f0 = f0/FS *2; % normalized freq
fac_third = 2^(1/3); % freq factor for third octave
K = floor(log(1/f0) / log(fac_third)); % number of bands

fprintf(1, 'band number (max. %i):\n', K);
for k=1:K
   fprintf(1, '%i ', k);
   f1 = f0*fac_third;  % upper freq of bandpass
   [b, a] = cheby1(2, 3, [f0 f1]); % chebyshev-type 1 filter design
   f0 = f1; %start freq for next band
   
   z1 = filter(b, a, y1); % filtering the two signals
   z2 = filter(b, a, y2);
   rms2 = sqrt(filter(1, lp, z2.*z2));% RMS value of sound 2
   rms1 = sqrt(epsi+filter(1, lp, z1.*z1));% with whitening
   %rms1 = 1.; % without whitening
   res = res + z1.*rms2./rms1; % add result to the output buffer
end
fprintf(1, '\n');
soundsc(res,FS)
DAFx_out_norm = .99* res/max(abs(res)); % scale for wav output
wavwrite(DAFx_out_norm, FS, 'CrossCV')
