% M-file 9.7
% LPCCrossSynhesis.m 
%
% cross-synthesis with LPC

clear; 

%----- USER DATA -----
[DAFx_in1,FS] = wavread('moore_guitar.wav'); % sound 1: excitation
DAFx_in2 = wavread('toms_diner.wav'); % sound 2: spectral env.
long     = 400; % block length for calculation of coefficients
hopsize  = 160; % hop size (is 160)
order    = 20 % order of the LPC
order1   = 6 % order for the excitation

%----- initializations -----
ly = min(length(DAFx_in1), length(DAFx_in2));
DAFx_in1 = [zeros(order, 1); DAFx_in1; ...
   zeros(order-mod(ly,hopsize),1)] / max(abs(DAFx_in1));
DAFx_in2 = [zeros(order, 1); DAFx_in2; ...
   zeros(order-mod(ly,hopsize),1)] / max(abs(DAFx_in2));
DAFx_out = zeros(ly,1); % result sound
exc      = zeros(ly,1); % excitation sound
w        = hanningz(long); % window
N_frames = floor((ly-order-long)/hopsize); % number of frames

%----- Cross-synthesis -----
tic
for j=1:N_frames
   k        = order + hopsize*(j-1); % offset of the buffer
   [A, g]   = calc_lpc(DAFx_in2(k+1:k+long).*w, order);
   [A1, g1] = calc_lpc(DAFx_in1(k+1:k+long).*w, order1);
   % IMPORTANT function "lpc" does not give correct results for MATLAB 6 !!!

   gain(j) = g; %
   ae = - A(2:order+1); % LPC coeff. of excitation
   for n=1:hopsize
      excitation1   = (A1/g1) * DAFx_in1(k+n:-1:k+n-order1);
      exc(k+n)      = excitation1;
      DAFx_out(k+n) = ae*DAFx_out(k+n-1:-1:k+n-order)+g*excitation1;
   end
end
toc
%----- output -----
DAFx_out = DAFx_out(order+1:length(DAFx_out)) / max(abs(DAFx_out));
soundsc(DAFx_out, FS)
DAFx_out_norm = .99* DAFx_out/max(abs(DAFx_out)); % scale for wav output
wavwrite(DAFx_out_norm, FS, 'CrossLPC')

