% M-file 9.10
% UX_fmove_cepstrum.m

clear; clf;

%----- USER DATA -----
[DAFx_in, SR] = wavread('la.wav'); % sound file
warping_coef  = 2.0;
n1            = 512;            % analysis hop size
n2            = n1;             % synthesis hop size
WindowLength  = 2048;           % window length
w1            = hanningz(WindowLength); % analysis window
w2            = w1;             % syhnthesis window
order         = 50;             % cut quefrency

%----- initialisations -----
WindowLength2 = WindowLength/2;
L             = length(DAFx_in);
DAFx_in       = [zeros(WindowLength, 1); DAFx_in; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in));
DAFx_out      = zeros(L,1);
t             = 1 + floor((0:WindowLength-1)*warping_coef); % apply the warping
lmax          = max(WindowLength,t(WindowLength));

tic
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
pin  = 0;
pout = 0;
pend = L - WindowLength;
%pin  = 6500;               % for plot
%pend = pin + WindowLength; % for plot
while pin<pend
   grain  = DAFx_in(pin+1:pin+WindowLength).* w1;
%===========================================
   f      = fft(grain)/WindowLength2;            % spectrum of grain
   flogs  = 20*log10(0.00001+abs(f));    % log|X(k)|

   grain1 = DAFx_in(pin+t).* w1;         % linear interpolation of grain
   f1     = fft(grain1)/WindowLength2;           % spectrum of interpolated grain
   flogs1 = 20*log10(0.00001 + abs(f1)); % log|X1(k)|
   flog   = log(0.00001+abs(f1)) - log(0.00001+abs(f));
   cep    = ifft(flog);                  % cepstrum
   cep_coupe = [cep(1)/2; cep(2:order); zeros(WindowLength-order,1)];

   corr   = exp(2*real(fft(cep_coupe))); % spectral shape
   grain  = (real(ifft(f.*corr))).*w2;

   fout   = fft(grain);
   flogs2 = 20*log10(0.00001+abs(fout));

   %----- figures for real-time spectral shape up to FS/2 -----
   subplot(3,1,1);  plot((1:WindowLength2/2)*44100/WindowLength, flogs(1:WindowLength2/2));
   title('a) original spectrum'); drawnow;
   subplot(3,1,2);  plot((1:WindowLength2/2)*44100/WindowLength, flogs1(1:WindowLength2/2));
   title('b) spectrum of time-scaled signal');
   subplot(3,1,3);  plot((1:WindowLength2/2)*44100/WindowLength, flogs2(1:WindowLength2/2));
   title('c) formant changed spectrum');
   xlabel('f in Hz \rightarrow');
   drawnow
% ===========================================
   DAFx_out(pout+1:pout+WindowLength) = DAFx_out(pout+1:pout+WindowLength) + grain;
   pin  = pin + n1;
   pout = pout + n2;
end
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
toc

%----- listening and saving the output -----
soundsc(DAFx_out, SR);
DAFx_out_norm = .99* DAFx_out/max(abs(DAFx_out)); % scale for wav output
wavwrite(DAFx_out_norm, SR, 'la_fmove.wav');
