% M-file 9.14
% UX_interp_move.m

clear; 

%----- USER DATA -----
[DAFx_in, SR] = wavread('la.wav'); % sound file
n1            = 400;  % analysis hop size, try n1=400 (pitch up) or 150 (pitch down)
n2            = 256;  % synthesis hop size, keep it a divisor of WindowLength (256 is pretty good)
WindowLength  = 2048; % window length
w1            = hanningz(WindowLength); % analysis window
w2            = w1;   % synthesis window
order         = 50;   % cut quefrency

%----- initialisations -----
ral           = n2/n1
WindowLength2 = WindowLength/2;
% for linear interpolation of a grain of length WindowLength
lx            = floor(WindowLength*n1/n2);
x             = 1 + (0:WindowLength-1)'*lx/WindowLength;
ix            = floor(x);
ix1           = ix + 1;
dx            = x - ix;
dx1           = 1 - dx;
warping_coef  = n1/n2;
lmax          = max(WindowLength,lx)
L             = length(DAFx_in);
DAFx_in       = [zeros(WindowLength, 1); DAFx_in] / max(abs(DAFx_in));
DAFx_out      = zeros(ceil(ral*length(DAFx_in)),1);

tic
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
pin  = 0;
pout = 0;
pend = L - lmax;

while pin<pend
%===========================================
   %----- interpolated grain
   grain1 = (DAFx_in(pin+ix).*dx1 + DAFx_in(pin+ix1).*dx).* w1;
   f1     = fft(grain1)/WindowLength2;
   %----- reference grain for formant matching
   grain2 = DAFx_in(pin+1:pin+WindowLength).* w1;
   f2     = fft(grain2)/WindowLength2;
   %----- correcting factor for spectral shape
   flog   = log(0.00001+abs(f2))-log(0.00001+abs(f1));
   cep    = ifft(flog);
   cep_coupe = [cep(1)/2; cep(2:order); zeros(WindowLength-order,1)];
   corr   = exp(2*real(fft(cep_coupe)));
   %----- so now make the formant move
   grain  = fftshift(real(ifft(f1.*corr))).*w2;
   % plot(grain);drawnow;
%===========================================
   DAFx_out(pout+1:pout+WindowLength) = DAFx_out(pout+1:pout+WindowLength) + grain;
   pin    = pin + n1;
   pout   = pout + n2;
end
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
toc

%----- listening and saving the output -----
DAFx_in = DAFx_in(WindowLength+1:WindowLength+L);
DAFx_out = .99* DAFx_out(WindowLength+1:length(DAFx_out)) / max(abs(DAFx_out));
soundsc(DAFx_out, SR);
wavwrite(DAFx_out, SR, 'la_interp_move.wav');
