% M-file 9.12
% UX_spectral_interp.m

% k (spectral mix)
% =0 ->x1 =1->x2 in between spectral interpolation
% in this example k is calculated at every step
% so we move from sound 1 to sound 2;

clear; 

%----- USER DATA -----
[DAFx_in1,SR] = wavread('claire_oubli_voix.WAV');  % sound 1 
DAFx_in2      = wavread('claire_oubli_flute.WAV'); % sound 2
n1            = 512;            % analysis hop size
n2            = n1;             % synthesis hop size
WindowLength  = 2048;           % window length
w1            = hanningz(WindowLength); % analysis window
w2            = w1;             % syhnthesis window
cut           = 50              % cut quefrency

%----- initialisations -----
L             = min(length(DAFx_in1), length(DAFx_in2));
DAFx_in1      = [zeros(WindowLength, 1); DAFx_in1; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in1));
DAFx_in2      = [zeros(WindowLength, 1); DAFx_in2; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in2));
DAFx_out      = zeros(length(DAFx_in1),1);

tic
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
pin  = 0;
pout = 0;
pend = L - WindowLength;
while pin<pend
   %----- here is the k factor varies between 0 and 1
   k      = pin/pend;               % spectral mix
   kp     = 1-k;

   grain1 = DAFx_in1(pin+1:pin+WindowLength).* w1;
   grain2 = DAFx_in2(pin+1:pin+WindowLength).* w1;
%===========================================
   f1    = fft(fftshift(grain1));
   flog  = log(0.00001+abs(f1));
   cep   = fft(flog);
   cep_coupe   = [cep(1)/2; cep(2:cut); zeros(WindowLength-cut,1)];
   flog_coupe1 = 2*real(ifft(cep_coupe));
   spec1 = exp(flog_coupe1);        % spectral shape of sound 1
   
   f2    = fft(fftshift(grain2));
   flog  = log(0.00001+abs(f2));
   cep   = fft(flog);
   cep_coupe   = [cep(1)/2; cep(2:cut); zeros(WindowLength-cut,1)];
   flog_coupe2 = 2*real(ifft(cep_coupe));
   spec2 = exp(flog_coupe2);        % spectral shape of sound 2
   
   %----- here we interpolate the spectral shapes in dBs
   spec  = exp(kp*flog_coupe1+k*flog_coupe2);
   
   ft    = (kp*f1./spec1+k*f2./spec2).*spec;
   grain = fftshift(real(ifft(ft))).*w2;
%===========================================
   DAFx_out(pout+1:pout+WindowLength) = DAFx_out(pout+1:pout+WindowLength) + grain;
   pin  = pin + n1;
   pout = pout + n2;
end
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
toc

%----- listening and saving the output -----
%DAFx_in = DAFx_in1(WindowLength+1:WindowLength+L);
DAFx_out = DAFx_out(WindowLength+1:WindowLength+L) / max(abs(DAFx_out));
soundsc(DAFx_out, SR);
wavwrite(DAFx_out, SR, 'spec_interp.wav');
